@@ -36,19 +36,34 @@ from typing import (
     Tuple,
     Type,
     TypeVar,
+    TYPE_CHECKING,
     Union,
 )
 
-from typing_extensions import NoReturn
+try:
+    from typing import NoReturn
+except ImportError:
+    from typing_extensions import NoReturn
 
 HAVE_ANNOTATED = sys.version_info >= (3, 7, 0)
 
-if HAVE_ANNOTATED:
+# This is a messy, type-wise, because we not only have two potentially conflicting imports here
+# but we also define our own versions in the else block in case we operate on Python 3.5
+# or 3.6 which didn't get Annotated support in get_type_hints(). The easiest way to make mypy
+# happy here is to tell it the versions from typing_extensions are canonical. Since this
+# typing_extensions import is only for mypy it'll work even without typing_extensions actually
+# installed so all's good.
+if TYPE_CHECKING:
+    from typing_extensions import _AnnotatedAlias, Annotated, get_type_hints
+elif HAVE_ANNOTATED:
     # Ignoring errors here as typing_extensions stub doesn't know about those things yet
-    from typing_extensions import _AnnotatedAlias, Annotated, get_type_hints  # type: ignore
+    try:
+        from typing import _AnnotatedAlias, Annotated, get_type_hints
+    except ImportError:
+        from typing_extensions import _AnnotatedAlias, Annotated, get_type_hints
 else:
 
-    class Annotated:  # type: ignore
+    class Annotated:
         pass
 
     from typing import get_type_hints as _get_type_hints
@@ -66,7 +81,7 @@ TYPING353 = hasattr(Union[str, int], '__origin__')
 
 
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.18.3'
+__version__ = '0.18.4'
 __version_tag__ = ''
 
 log = logging.getLogger('injector')
@@ -110,7 +125,7 @@ if HAVE_ANNOTATED:
     InjectT = TypeVar('InjectT')
     Inject = Annotated[InjectT, _inject_marker]
     """An experimental way to declare injectable dependencies utilizing a `PEP 593`_ implementation
-    in `typing_extensions`.
+    in Python 3.9 and backported to Python 3.7+ in `typing_extensions`.
 
     Those two declarations are equivalent::
 
@@ -150,7 +165,7 @@ if HAVE_ANNOTATED:
 
     NoInject = Annotated[InjectT, _noinject_marker]
     """An experimental way to declare noninjectable dependencies utilizing a `PEP 593`_ implementation
-    in `typing_extensions`.
+    in Python 3.9 and backported to Python 3.7+ in `typing_extensions`.
 
     Since :func:`inject` declares all function's parameters to be injectable there needs to be a way
     to opt out of it. This has been provided by :func:`noninjectable` but `noninjectable` suffers from
@@ -305,7 +320,7 @@ class CallableProvider(Provider):
         providing
         providing
         False
-        """
+    """
 
     def __init__(self, callable: Callable[..., T]):
         self._callable = callable
@@ -1188,9 +1203,7 @@ def _infer_injected_bindings(callable: Callable, only_explicit_bindings: bool) -
 
         if only_explicit_bindings and _inject_marker not in metadata or _noinject_marker in metadata:
             del bindings[k]
-            break
-
-        if _is_specialization(v, Union):
+        elif _is_specialization(v, Union):
             # We don't treat Optional parameters in any special way at the moment.
             if TYPING353:
                 union_members = v.__args__
@@ -1481,19 +1494,19 @@ def _describe(c: Any) -> str:
 class ProviderOf(Generic[T]):
     """Can be used to get a provider of an interface, for example:
 
-        >>> def provide_int():
-        ...     print('providing')
-        ...     return 123
-        >>>
-        >>> def configure(binder):
-        ...     binder.bind(int, to=provide_int)
-        >>>
-        >>> injector = Injector(configure)
-        >>> provider = injector.get(ProviderOf[int])
-        >>> value = provider.get()
-        providing
-        >>> value
-        123
+    >>> def provide_int():
+    ...     print('providing')
+    ...     return 123
+    >>>
+    >>> def configure(binder):
+    ...     binder.bind(int, to=provide_int)
+    >>>
+    >>> injector = Injector(configure)
+    >>> provider = injector.get(ProviderOf[int])
+    >>> value = provider.get()
+    providing
+    >>> value
+    123
     """
 
     def __init__(self, injector: Injector, interface: Type[T]):
